@@ -28,6 +28,7 @@ SUPPORTED_ISOLATION = ['filesystem/posix', 'filesystem/linux',
                        'cgroups/mem', 'docker/runtime',
                        'namespaces/pid']
 SUPPORTED_IMAGE_PROVIDERS = ['docker', 'appc']
+SUPPORTED_SWARM_STRATEGY = ['spread', 'binpack', 'random']
 
 
 def validate_image(cli, image):
@@ -48,7 +49,7 @@ def validate_image(cli, image):
 def validate_flavor(cli, flavor):
     """Validate flavor.
 
-    If flavor is None, skip the validation and using the default value
+    If flavor is None, skip the validation and use the default value
     from the heat template.
     """
 
@@ -166,18 +167,41 @@ def validate_labels_executor_env_variables(labels):
         raise exception.InvalidParameterValue(err)
 
 
-def validate_os_resources(context, cluster_template):
+def validate_labels_strategy(labels):
+    """Validate swarm_strategy"""
+    swarm_strategy = list(labels.get('swarm_strategy', "").split())
+    unsupported_strategy = set(swarm_strategy) - set(
+        SUPPORTED_SWARM_STRATEGY)
+    if (len(unsupported_strategy) > 0):
+        raise exception.InvalidParameterValue(_(
+            'property "labels/swarm_strategy" with value '
+            '"%(strategy)s" is not supported, supported values are: '
+            '%(supported_strategies)s') % {
+                'strategy': ' '.join(list(unsupported_strategy)),
+                'supported_strategies': ', '.join(
+                    SUPPORTED_SWARM_STRATEGY + ['unspecified'])})
+
+
+def validate_os_resources(context, cluster_template, cluster=None):
     """Validate ClusterTemplate's OpenStack Resources"""
 
     cli = clients.OpenStackClients(context)
 
     for attr, validate_method in validators.items():
-        if attr in cluster_template and cluster_template[attr] is not None:
+        if cluster and attr in cluster and cluster[attr]:
+            if attr != 'labels':
+                validate_method(cli, cluster[attr])
+            else:
+                validate_method(cluster[attr])
+        elif attr in cluster_template and cluster_template[attr] is not None:
             if attr != 'labels':
                 validate_method(cli, cluster_template[attr])
             else:
                 validate_method(cluster_template[attr])
 
+    if cluster:
+        validate_keypair(cli, cluster['keypair'])
+
 
 def validate_master_count(cluster, cluster_template):
     if cluster['master_count'] > 1 and \
@@ -190,7 +214,6 @@ def validate_master_count(cluster, cluster_template):
 validators = {'image_id': validate_image,
               'flavor_id': validate_flavor,
               'master_flavor_id': validate_flavor,
-              'keypair_id': validate_keypair,
               'external_network_id': validate_external_network,
               'fixed_network': validate_fixed_network,
               'labels': validate_labels}
@@ -199,4 +222,5 @@ labels_validators = {'mesos_slave_isolation': validate_labels_isolation,
                      'mesos_slave_image_providers':
                      validate_labels_image_providers,
                      'mesos_slave_executor_env_variables':
-                     validate_labels_executor_env_variables}
+                     validate_labels_executor_env_variables,
+                     'swarm_strategy': validate_labels_strategy}
