@@ -64,8 +64,8 @@ class ClusterTemplate(base.APIBase):
     """The DNS nameserver address"""
 
     keypair_id = wsme.wsattr(wtypes.StringType(min_length=1, max_length=255),
-                             mandatory=True)
-    """The name or id of the nova ssh keypair"""
+                             default=None)
+    """The name of the nova ssh keypair"""
 
     external_network_id = wtypes.StringType(min_length=1, max_length=255)
     """The external network to attach to the Cluster"""
@@ -121,15 +121,14 @@ class ClusterTemplate(base.APIBase):
     public = wsme.wsattr(types.boolean, default=False)
     """Indicates whether the ClusterTemplate is public or not."""
 
-    server_type = wsme.wsattr(wtypes.StringType(min_length=1,
-                                                max_length=255),
+    server_type = wsme.wsattr(wtypes.Enum(str, *fields.ServerType.ALL),
                               default='vm')
     """Server type for this ClusterTemplate """
 
     insecure_registry = wtypes.StringType(min_length=1, max_length=255)
     """Insecure registry URL when creating a ClusterTemplate """
 
-    docker_storage_driver = wtypes.Enum(str, *fields.DockerStorageDriver.ALL)
+    docker_storage_driver = wtypes.StringType(min_length=1, max_length=255)
     """Docker storage driver"""
 
     master_lb_enabled = wsme.wsattr(types.boolean, default=False)
@@ -140,6 +139,12 @@ class ClusterTemplate(base.APIBase):
     floating_ip_enabled = wsme.wsattr(types.boolean, default=True)
     """Indicates whether created clusters should have a floating ip or not."""
 
+    project_id = wsme.wsattr(wtypes.text, readonly=True)
+    """Project id of the cluster belongs to"""
+
+    user_id = wsme.wsattr(wtypes.text, readonly=True)
+    """User id of the cluster belongs to"""
+
     def __init__(self, **kwargs):
         self.fields = []
         for field in objects.ClusterTemplate.fields:
@@ -250,6 +255,23 @@ class ClusterTemplatesController(base.Controller):
                                           sort_key, sort_dir,
                                           resource_url=None):
 
+        context = pecan.request.context
+        if context.is_admin:
+            if resource_url == '/'.join(['clustertemplates', 'detail']):
+                policy.enforce(context, "clustertemplate:detail_all_projects",
+                               action="clustertemplate:detail_all_projects")
+            else:
+                policy.enforce(context, "clustertemplate:get_all_all_projects",
+                               action="clustertemplate:get_all_all_projects")
+            # TODO(flwang): Instead of asking an extra 'all_project's
+            # parameter, currently the design is allowing admin user to list
+            # all clusters from all projects. But the all_tenants is one of
+            # the condition to do project filter in DB API. And it's also used
+            # by periodic tasks. So the could be removed in the future and
+            # a new parameter 'project_id' would be added so that admin user
+            # can list clusters for a particular project.
+            context.all_tenants = True
+
         limit = api_utils.validate_limit(limit)
         sort_dir = api_utils.validate_sort_dir(sort_dir)
 
@@ -318,15 +340,31 @@ class ClusterTemplatesController(base.Controller):
         ClusterTemplate.
         """
         context = pecan.request.context
+
+        if context.is_admin:
+            policy.enforce(context, "clustertemplate:get_one_all_projects",
+                           action="clustertemplate:get_one_all_projects")
+            # TODO(flwang): Instead of asking an extra 'all_project's
+            # parameter, currently the design is allowing admin user to list
+            # all clusters from all projects. But the all_tenants is one of
+            # the condition to do project filter in DB API. And it's also used
+            # by periodic tasks. So the could be removed in the future and
+            # a new parameter 'project_id' would be added so that admin user
+            # can list clusters for a particular project.
+            context.all_tenants = True
+
         cluster_template = api_utils.get_resource('ClusterTemplate',
                                                   cluster_template_ident)
+
         if not cluster_template.public:
-            policy.enforce(context, 'clustertemplate:get', cluster_template,
+            policy.enforce(context, 'clustertemplate:get',
+                           cluster_template.as_dict(),
                            action='clustertemplate:get')
 
         return ClusterTemplate.convert_with_links(cluster_template)
 
     @expose.expose(ClusterTemplate, body=ClusterTemplate, status_code=201)
+    @validation.enforce_server_type()
     @validation.enforce_network_driver_types_create()
     @validation.enforce_volume_driver_types_create()
     @validation.enforce_volume_storage_size_create()
@@ -383,7 +421,8 @@ class ClusterTemplatesController(base.Controller):
         context = pecan.request.context
         cluster_template = api_utils.get_resource('ClusterTemplate',
                                                   cluster_template_ident)
-        policy.enforce(context, 'clustertemplate:update', cluster_template,
+        policy.enforce(context, 'clustertemplate:update',
+                       cluster_template.as_dict(),
                        action='clustertemplate:update')
         try:
             cluster_template_dict = cluster_template.as_dict()
@@ -427,6 +466,7 @@ class ClusterTemplatesController(base.Controller):
         context = pecan.request.context
         cluster_template = api_utils.get_resource('ClusterTemplate',
                                                   cluster_template_ident)
-        policy.enforce(context, 'clustertemplate:delete', cluster_template,
+        policy.enforce(context, 'clustertemplate:delete',
+                       cluster_template.as_dict(),
                        action='clustertemplate:delete')
         cluster_template.destroy()
