@@ -19,7 +19,6 @@ Includes decorator for re-raising Magnum-type exceptions.
 """
 
 import functools
-import json
 import sys
 
 from keystoneclient import exceptions as keystone_exceptions
@@ -27,13 +26,13 @@ from oslo_config import cfg
 from oslo_log import log as logging
 import six
 
+import magnum.conf
 from magnum.i18n import _
-from magnum.i18n import _LE
 
 
 LOG = logging.getLogger(__name__)
 
-CONF = cfg.CONF
+CONF = magnum.conf.CONF
 
 try:
     CONF.import_opt('fatal_exception_format_errors',
@@ -92,8 +91,8 @@ class MagnumException(Exception):
         except Exception:
             # kwargs doesn't match a variable in the message
             # log the issue and the kwargs
-            LOG.exception(_LE('Exception in string format operation, '
-                              'kwargs: %s') % kwargs)
+            LOG.exception('Exception in string format operation, '
+                          'kwargs: %s', kwargs)
             try:
                 if CONF.fatal_exception_format_errors:
                     raise
@@ -121,15 +120,20 @@ class MagnumException(Exception):
 
 class ObjectNotFound(MagnumException):
     message = _("The %(name)s %(id)s could not be found.")
+    code = 404
+
+
+class ProjectNotFound(ObjectNotFound):
+    message = _("The %(name)s %(id)s could not be found.")
 
 
 class ResourceNotFound(ObjectNotFound):
     message = _("The %(name)s resource %(id)s could not be found.")
-    code = 404
 
 
 class AuthorizationFailure(MagnumException):
     message = _("%(client)s connection failed. %(message)s")
+    code = 403
 
 
 class Invalid(MagnumException):
@@ -188,7 +192,7 @@ class Conflict(MagnumException):
     code = 409
 
 
-class ApiVersionsIntersect(MagnumException):
+class ApiVersionsIntersect(Invalid):
     message = _("Version of %(name)s %(min_ver)s %(max_ver)s intersects "
                 "with another versions.")
 
@@ -216,19 +220,10 @@ class InvalidMAC(Invalid):
     message = _("Expected a MAC address but received %(mac)s.")
 
 
-class ConfigInvalid(MagnumException):
+class ConfigInvalid(Invalid):
     message = _("Invalid configuration file. %(error_msg)s")
 
 
-class SSHConnectFailed(MagnumException):
-    message = _("Failed to establish SSH connection to host %(host)s.")
-
-
-class FileSystemNotSupported(MagnumException):
-    message = _("Failed to create a file system. "
-                "File system %(fs)s is not supported.")
-
-
 class ClusterTemplateNotFound(ResourceNotFound):
     message = _("ClusterTemplate %(clustertemplate)s could not be found.")
 
@@ -254,74 +249,20 @@ class ClusterAlreadyExists(Conflict):
     message = _("A cluster with UUID %(uuid)s already exists.")
 
 
-class ContainerNotFound(ResourceNotFound):
-    message = _("Container %(container)s could not be found.")
-
-
-class ContainerAlreadyExists(Conflict):
-    message = _("A container with UUID %(uuid)s already exists.")
-
-
-class PodNotFound(ResourceNotFound):
-    message = _("Pod %(pod)s could not be found.")
-
-
-class PodAlreadyExists(Conflict):
-    message = _("A pod with UUID %(uuid)s already exists.")
-
-
-class PodListNotFound(ResourceNotFound):
-    message = _("Pod list could not be found for Cluster %(cluster_uuid)s.")
-
-
-class PodCreationFailed(Invalid):
-    message = _("Pod creation failed in Cluster %(cluster_uuid)s.")
-
-
-class ServiceNotFound(ResourceNotFound):
-    message = _("Service %(service)s could not be found.")
-
-
-class ServiceAlreadyExists(Conflict):
-    message = _("A service with UUID %(uuid)s already exists.")
-
-
-class ServiceListNotFound(ResourceNotFound):
-    message = _("Service list could not be found for Cluster "
-                "%(cluster_uuid)s.")
-
-
-class ServiceCreationFailed(Invalid):
-    message = _("Service creation failed for Cluster %(cluster_uuid)s.")
-
-
-class ContainerException(Exception):
-    pass
-
-
 class NotSupported(MagnumException):
     message = _("%(operation)s is not supported.")
     code = 400
 
 
-class ClusterTypeNotSupported(MagnumException):
+class ClusterTypeNotSupported(NotSupported):
     message = _("Cluster type (%(server_type)s, %(os)s, %(coe)s)"
                 " not supported.")
 
 
-class ClusterTypeNotEnabled(MagnumException):
-    message = _("Cluster type (%(server_type)s, %(os)s, %(coe)s)"
-                " not enabled.")
-
-
-class RequiredParameterNotProvided(MagnumException):
+class RequiredParameterNotProvided(Invalid):
     message = _("Required parameter %(heat_param)s not provided.")
 
 
-class Urllib2InvalidScheme(MagnumException):
-    message = _("The urllib2 URL %(url)s has an invalid scheme.")
-
-
 class OperationInProgress(Invalid):
     message = _("Cluster %(cluster_name)s already has an operation in "
                 "progress.")
@@ -333,7 +274,7 @@ class ImageNotFound(ResourceNotFound):
     code = 400
 
 
-class ImageNotAuthorized(MagnumException):
+class ImageNotAuthorized(NotAuthorized):
     message = _("Not authorized for image %(image_id)s.")
 
 
@@ -343,19 +284,6 @@ class OSDistroFieldNotFound(ResourceNotFound):
     code = 400
 
 
-class KubernetesAPIFailed(MagnumException):
-    def __init__(self, message=None, err=None, **kwargs):
-        if err:
-            if err.body:
-                message = json.loads(err.body)['message']
-            else:
-                message = err.reason
-            self.__class__.code = err.status
-        else:
-            self.__class__.code = kwargs.get('code')
-        super(KubernetesAPIFailed, self).__init__(message, **kwargs)
-
-
 class X509KeyPairNotFound(ResourceNotFound):
     message = _("A key pair %(x509keypair)s could not be found.")
 
@@ -384,11 +312,11 @@ class MagnumServiceAlreadyExists(Conflict):
     message = _("A magnum service with ID %(id)s already exists.")
 
 
-class UnsupportedK8sQuantityFormat(MagnumException):
+class UnsupportedK8sQuantityFormat(Invalid):
     message = _("Unsupported quantity format for k8s cluster.")
 
 
-class UnsupportedDockerQuantityFormat(MagnumException):
+class UnsupportedDockerQuantityFormat(Invalid):
     message = _("Unsupported quantity format for Swarm cluster.")
 
 
@@ -427,6 +355,14 @@ class QuotaAlreadyExists(Conflict):
                 "for resource %(resource)s.")
 
 
+class QuotaNotFound(ResourceNotFound):
+    message = _("Quota could not be found: %(msg)s")
+
+
+class ResourceLimitExceeded(NotAuthorized):
+    message = _('Resource limit exceeded: %(msg)s')
+
+
 class RegionsListFailed(MagnumException):
     message = _("Failed to list regions.")
 
@@ -438,3 +374,11 @@ class TrusteeOrTrustToClusterFailed(MagnumException):
 
 class CertificatesToClusterFailed(MagnumException):
     message = _("Failed to create certificates for Cluster: %(cluster_uuid)s")
+
+
+class FederationNotFound(ResourceNotFound):
+    message = _("Federation %(federation)s could not be found.")
+
+
+class FederationAlreadyExists(Conflict):
+    message = _("A federation with UUID %(uuid)s already exists.")
