@@ -26,6 +26,7 @@ class ClusterStatus(fields.Enum):
     DELETE_FAILED = 'DELETE_FAILED'
     DELETE_COMPLETE = 'DELETE_COMPLETE'
     RESUME_COMPLETE = 'RESUME_COMPLETE'
+    RESUME_FAILED = 'RESUME_FAILED'
     RESTORE_COMPLETE = 'RESTORE_COMPLETE'
     ROLLBACK_IN_PROGRESS = 'ROLLBACK_IN_PROGRESS'
     ROLLBACK_FAILED = 'ROLLBACK_FAILED'
@@ -37,12 +38,12 @@ class ClusterStatus(fields.Enum):
     ALL = (CREATE_IN_PROGRESS, CREATE_FAILED, CREATE_COMPLETE,
            UPDATE_IN_PROGRESS, UPDATE_FAILED, UPDATE_COMPLETE,
            DELETE_IN_PROGRESS, DELETE_FAILED, DELETE_COMPLETE,
-           RESUME_COMPLETE, RESTORE_COMPLETE, ROLLBACK_IN_PROGRESS,
-           ROLLBACK_FAILED, ROLLBACK_COMPLETE, SNAPSHOT_COMPLETE,
-           CHECK_COMPLETE, ADOPT_COMPLETE)
+           RESUME_COMPLETE, RESUME_FAILED, RESTORE_COMPLETE,
+           ROLLBACK_IN_PROGRESS, ROLLBACK_FAILED, ROLLBACK_COMPLETE,
+           SNAPSHOT_COMPLETE, CHECK_COMPLETE, ADOPT_COMPLETE)
 
     STATUS_FAILED = (CREATE_FAILED, UPDATE_FAILED,
-                     DELETE_FAILED, ROLLBACK_FAILED)
+                     DELETE_FAILED, ROLLBACK_FAILED, RESUME_FAILED)
 
     def __init__(self):
         super(ClusterStatus, self).__init__(valid_values=ClusterStatus.ALL)
@@ -62,25 +63,37 @@ class ContainerStatus(fields.Enum):
 
 class ClusterType(fields.Enum):
     ALL = (
-        KUBERNETES, SWARM, MESOS,
+        KUBERNETES, SWARM, MESOS, DCOS, SWARM_MODE,
     ) = (
-        'kubernetes', 'swarm', 'mesos',
+        'kubernetes', 'swarm', 'mesos', 'dcos', 'swarm-mode',
     )
 
     def __init__(self):
         super(ClusterType, self).__init__(valid_values=ClusterType.ALL)
 
 
-class DockerStorageDriver(fields.Enum):
+class QuotaResourceName(fields.Enum):
     ALL = (
-        DEVICEMAPPER, OVERLAY,
+        CLUSTER,
     ) = (
-        'devicemapper', 'overlay',
+        'Cluster',
     )
 
     def __init__(self):
-        super(DockerStorageDriver, self).__init__(
-            valid_values=DockerStorageDriver.ALL)
+        super(QuotaResourceName, self).__init__(
+            valid_values=QuotaResourceName.ALL)
+
+
+class ServerType(fields.Enum):
+    ALL = (
+        VM, BM,
+    ) = (
+        'vm', 'bm',
+    )
+
+    def __init__(self):
+        super(ServerType, self).__init__(
+            valid_values=ServerType.ALL)
 
 
 class MagnumServiceState(fields.Enum):
@@ -131,5 +144,5 @@ class ClusterTypeField(fields.BaseEnumField):
     AUTO_TYPE = ClusterType()
 
 
-class DockerStorageDriverField(fields.BaseEnumField):
-    AUTO_TYPE = DockerStorageDriver()
+class ServerTypeField(fields.BaseEnumField):
+    AUTO_TYPE = ServerType()
