@@ -16,17 +16,21 @@ import mock
 from oslo_config import cfg
 from oslo_utils import timeutils
 from oslo_utils import uuidutils
+from wsme import types as wtypes
 
 from magnum.api import attr_validator
 from magnum.api.controllers.v1 import cluster as api_cluster
 from magnum.common import exception
 from magnum.conductor import api as rpcapi
+import magnum.conf
 from magnum import objects
 from magnum.tests import base
 from magnum.tests.unit.api import base as api_base
 from magnum.tests.unit.api import utils as apiutils
 from magnum.tests.unit.objects import utils as obj_utils
 
+CONF = magnum.conf.CONF
+
 
 class TestClusterObject(base.TestCase):
     def test_cluster_init(self):
@@ -39,6 +43,15 @@ class TestClusterObject(base.TestCase):
         self.assertEqual(1, cluster.master_count)
         self.assertEqual(60, cluster.create_timeout)
 
+        # test unset value for cluster_template_id
+        cluster.cluster_template_id = wtypes.Unset
+        self.assertEqual(wtypes.Unset, cluster.cluster_template_id)
+
+        # test backwards compatibility of bay fields with new objects
+        cluster_dict['create_timeout'] = 15
+        cluster = api_cluster.Cluster(**cluster_dict)
+        self.assertEqual(15, cluster.create_timeout)
+
 
 class TestListCluster(api_base.FunctionalTest):
     _cluster_attrs = ("name", "cluster_template_id", "node_count", "status",
@@ -115,6 +128,33 @@ class TestListCluster(api_base.FunctionalTest):
         self.assertEqual('application/json', response.content_type)
         self.assertTrue(response.json['errors'])
 
+    def test_get_one_by_uuid(self):
+        temp_uuid = uuidutils.generate_uuid()
+        obj_utils.create_test_cluster(self.context, uuid=temp_uuid)
+        response = self.get_json(
+            '/clusters/%s' % temp_uuid)
+        self.assertEqual(temp_uuid, response['uuid'])
+
+    def test_get_one_by_uuid_not_found(self):
+        temp_uuid = uuidutils.generate_uuid()
+        response = self.get_json(
+            '/clusters/%s' % temp_uuid,
+            expect_errors=True)
+        self.assertEqual(404, response.status_int)
+        self.assertEqual('application/json', response.content_type)
+        self.assertTrue(response.json['errors'])
+
+    @mock.patch("magnum.common.policy.enforce")
+    @mock.patch("magnum.common.context.make_context")
+    def test_get_one_by_uuid_admin(self, mock_context, mock_policy):
+        temp_uuid = uuidutils.generate_uuid()
+        obj_utils.create_test_cluster(self.context, uuid=temp_uuid,
+                                      project_id=temp_uuid)
+        self.context.is_admin = True
+        response = self.get_json(
+            '/clusters/%s' % temp_uuid)
+        self.assertEqual(temp_uuid, response['uuid'])
+
     def test_get_one_by_name_multiple_cluster(self):
         obj_utils.create_test_cluster(self.context, name='test_cluster',
                                       uuid=uuidutils.generate_uuid())
@@ -140,6 +180,19 @@ class TestListCluster(api_base.FunctionalTest):
         self.assertEqual(cluster_list[-1].uuid,
                          response['clusters'][0]['uuid'])
 
+    @mock.patch("magnum.common.policy.enforce")
+    @mock.patch("magnum.common.context.make_context")
+    def test_get_all_with_all_projects(self, mock_context, mock_policy):
+        for id_ in range(4):
+            temp_uuid = uuidutils.generate_uuid()
+            obj_utils.create_test_cluster(self.context, id=id_,
+                                          uuid=temp_uuid,
+                                          project_id=id_)
+
+        self.context.is_admin = True
+        response = self.get_json('/clusters')
+        self.assertEqual(4, len(response['clusters']))
+
     def test_detail(self):
         cluster = obj_utils.create_test_cluster(self.context)
         response = self.get_json('/clusters/detail')
@@ -290,6 +343,19 @@ class TestPatch(api_base.FunctionalTest):
         self.assertEqual('application/json', response.content_type)
         self.assertEqual(404, response.status_code)
 
+    @mock.patch('oslo_utils.timeutils.utcnow')
+    def test_replace_ok_by_uuid_not_found(self, mock_utcnow):
+        uuid = uuidutils.generate_uuid()
+        test_time = datetime.datetime(2000, 1, 1, 0, 0)
+        mock_utcnow.return_value = test_time
+
+        response = self.patch_json('/clusters/%s' % uuid,
+                                   [{'path': '/cluster_id', 'value': uuid,
+                                     'op': 'replace'}],
+                                   expect_errors=True)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(404, response.status_code)
+
     def test_replace_cluster_template_id_failed(self):
         cluster_template = obj_utils.create_test_cluster_template(
             self.context,
@@ -370,6 +436,26 @@ class TestPatch(api_base.FunctionalTest):
         self.assertEqual(400, response.status_int)
         self.assertTrue(response.json['errors'])
 
+    def test_update_cluster_with_rollback_enabled(self):
+        response = self.patch_json(
+            '/clusters/%s/?rollback=True' % self.cluster_obj.uuid,
+            [{'path': '/node_count', 'value': 4,
+              'op': 'replace'}],
+            headers={'OpenStack-API-Version': 'container-infra 1.3'})
+
+        self.mock_cluster_update.assert_called_once_with(mock.ANY, True)
+        self.assertEqual(202, response.status_code)
+
+    def test_update_cluster_with_rollback_disabled(self):
+        response = self.patch_json(
+            '/clusters/%s/?rollback=False' % self.cluster_obj.uuid,
+            [{'path': '/node_count', 'value': 4,
+              'op': 'replace'}],
+            headers={'OpenStack-API-Version': 'container-infra 1.3'})
+
+        self.mock_cluster_update.assert_called_once_with(mock.ANY, False)
+        self.assertEqual(202, response.status_code)
+
     def test_remove_ok(self):
         response = self.get_json('/clusters/%s' % self.cluster_obj.uuid)
         self.assertIsNotNone(response['name'])
@@ -439,6 +525,27 @@ class TestPost(api_base.FunctionalTest):
         self.assertEqual(202, response.status_int)
         self.assertTrue(uuidutils.is_uuid_like(response.json['uuid']))
 
+    @mock.patch('oslo_utils.timeutils.utcnow')
+    def test_create_cluster_resource_limit_reached(self, mock_utcnow):
+        # override max_cluster_per_project to 1
+        CONF.set_override('max_clusters_per_project', 1, group='quotas')
+
+        bdict = apiutils.cluster_post_data()
+        test_time = datetime.datetime(2000, 1, 1, 0, 0)
+        mock_utcnow.return_value = test_time
+
+        # create first cluster
+        response = self.post_json('/clusters', bdict)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(202, response.status_int)
+        self.assertTrue(uuidutils.is_uuid_like(response.json['uuid']))
+
+        # now try to create second cluster and make sure it fails
+        response = self.post_json('/clusters', bdict, expect_errors=True)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(403, response.status_int)
+        self.assertTrue(response.json['errors'])
+
     def test_create_cluster_set_project_id_and_user_id(self):
         bdict = apiutils.cluster_post_data()
 
@@ -486,6 +593,14 @@ class TestPost(api_base.FunctionalTest):
         self.assertEqual(400, response.status_int)
         self.assertTrue(response.json['errors'])
 
+    def test_create_cluster_with_non_existent_cluster_template_name(self):
+        modelname = 'notfound'
+        bdict = apiutils.cluster_post_data(cluster_template_id=modelname)
+        response = self.post_json('/clusters', bdict, expect_errors=True)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(400, response.status_int)
+        self.assertTrue(response.json['errors'])
+
     def test_create_cluster_with_cluster_template_name(self):
         modelname = self.cluster_template.name
         bdict = apiutils.cluster_post_data(cluster_template_id=modelname)
@@ -531,102 +646,25 @@ class TestPost(api_base.FunctionalTest):
         self.assertEqual('application/json', response.content_type)
         self.assertEqual(202, response.status_int)
 
-    def test_create_cluster_with_invalid_long_name(self):
-        bdict = apiutils.cluster_post_data(name='x' * 243)
-        response = self.post_json('/clusters', bdict, expect_errors=True)
-        self.assertEqual('application/json', response.content_type)
-        self.assertEqual(400, response.status_int)
-        self.assertTrue(response.json['errors'])
+    def test_create_cluster_with_invalid_name(self):
+        invalid_names = ['x' * 243, '123456', '123456test_cluster',
+                         '-test_cluster', '.test_cluster', '_test_cluster', '']
+        for value in invalid_names:
+            bdict = apiutils.cluster_post_data(name=value)
+            response = self.post_json('/clusters', bdict, expect_errors=True)
+            self.assertEqual('application/json', response.content_type)
+            self.assertEqual(400, response.status_int)
+            self.assertTrue(response.json['errors'])
 
-    def test_create_cluster_with_invalid_integer_name(self):
-        bdict = apiutils.cluster_post_data(name='123456')
-        response = self.post_json('/clusters', bdict, expect_errors=True)
-        self.assertEqual('application/json', response.content_type)
-        self.assertEqual(400, response.status_int)
-        self.assertTrue(response.json['errors'])
-
-    def test_create_cluster_with_invalid_integer_str_name(self):
-        bdict = apiutils.cluster_post_data(name='123456test_cluster')
-        response = self.post_json('/clusters', bdict, expect_errors=True)
-        self.assertEqual('application/json', response.content_type)
-        self.assertEqual(400, response.status_int)
-        self.assertTrue(response.json['errors'])
-
-    def test_create_cluster_with_hyphen_invalid_at_start_name(self):
-        bdict = apiutils.cluster_post_data(name='-test_cluster')
-        response = self.post_json('/clusters', bdict, expect_errors=True)
-        self.assertEqual('application/json', response.content_type)
-        self.assertEqual(400, response.status_int)
-        self.assertTrue(response.json['errors'])
-
-    def test_create_cluster_with_period_invalid_at_start_name(self):
-        bdict = apiutils.cluster_post_data(name='.test_cluster')
-        response = self.post_json('/clusters', bdict, expect_errors=True)
-        self.assertEqual('application/json', response.content_type)
-        self.assertEqual(400, response.status_int)
-        self.assertTrue(response.json['errors'])
-
-    def test_create_cluster_with_underscore_invalid_at_start_name(self):
-        bdict = apiutils.cluster_post_data(name='_test_cluster')
-        response = self.post_json('/clusters', bdict, expect_errors=True)
-        self.assertEqual('application/json', response.content_type)
-        self.assertEqual(400, response.status_int)
-        self.assertTrue(response.json['errors'])
-
-    def test_create_cluster_with_valid_str_int_name(self):
-        bdict = apiutils.cluster_post_data(name='test_cluster123456')
-        response = self.post_json('/clusters', bdict, expect_errors=True)
-        self.assertEqual('application/json', response.content_type)
-        self.assertEqual(202, response.status_int)
-
-    def test_create_cluster_with_hyphen_valid_name(self):
-        bdict = apiutils.cluster_post_data(name='test-cluster')
-        response = self.post_json('/clusters', bdict, expect_errors=True)
-        self.assertEqual('application/json', response.content_type)
-        self.assertEqual(202, response.status_int)
-
-    def test_create_cluster_with_period_valid_name(self):
-        bdict = apiutils.cluster_post_data(name='test.cluster')
-        response = self.post_json('/clusters', bdict, expect_errors=True)
-        self.assertEqual('application/json', response.content_type)
-        self.assertEqual(202, response.status_int)
-
-    def test_create_cluster_with_period_at_end_valid_name(self):
-        bdict = apiutils.cluster_post_data(name='testcluster.')
-        response = self.post_json('/clusters', bdict, expect_errors=True)
-        self.assertEqual('application/json', response.content_type)
-        self.assertEqual(202, response.status_int)
-
-    def test_create_cluster_with_hyphen_at_end_valid_name(self):
-        bdict = apiutils.cluster_post_data(name='testcluster-')
-        response = self.post_json('/clusters', bdict, expect_errors=True)
-        self.assertEqual('application/json', response.content_type)
-        self.assertEqual(202, response.status_int)
-
-    def test_create_cluster_with_underscore_at_end_valid_name(self):
-        bdict = apiutils.cluster_post_data(name='testcluster_')
-        response = self.post_json('/clusters', bdict, expect_errors=True)
-        self.assertEqual('application/json', response.content_type)
-        self.assertEqual(202, response.status_int)
-
-    def test_create_cluster_with_mix_special_char_valid_name(self):
-        bdict = apiutils.cluster_post_data(name='test.-_cluster')
-        response = self.post_json('/clusters', bdict, expect_errors=True)
-        self.assertEqual('application/json', response.content_type)
-        self.assertEqual(202, response.status_int)
-
-    def test_create_cluster_with_capital_letter_start_valid_name(self):
-        bdict = apiutils.cluster_post_data(name='Testcluster')
-        response = self.post_json('/clusters', bdict, expect_errors=True)
-        self.assertEqual('application/json', response.content_type)
-        self.assertEqual(202, response.status_int)
-
-    def test_create_cluster_with_invalid_empty_name(self):
-        bdict = apiutils.cluster_post_data(name='')
-        response = self.post_json('/clusters', bdict, expect_errors=True)
-        self.assertEqual('application/json', response.content_type)
-        self.assertEqual(400, response.status_int)
-        self.assertTrue(response.json['errors'])
+    def test_create_cluster_with_valid_name(self):
+        valid_names = ['test_cluster123456', 'test-cluster', 'test.cluster',
+                       'testcluster.', 'testcluster-', 'testcluster_',
+                       'test.-_cluster', 'Testcluster']
+        for value in valid_names:
+            bdict = apiutils.cluster_post_data(name=value)
+            response = self.post_json('/clusters', bdict, expect_errors=True)
+            self.assertEqual('application/json', response.content_type)
+            self.assertEqual(202, response.status_int)
 
     def test_create_cluster_without_name(self):
         bdict = apiutils.cluster_post_data()
@@ -741,6 +779,125 @@ class TestPost(api_base.FunctionalTest):
         self.assertEqual('application/json', response.content_type)
         self.assertEqual(400, response.status_int)
 
+    def test_create_cluster_with_keypair(self):
+        bdict = apiutils.cluster_post_data()
+        bdict['keypair'] = 'keypair2'
+        response = self.post_json('/clusters', bdict)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(202, response.status_int)
+        cluster, timeout = self.mock_cluster_create.call_args
+        self.assertEqual('keypair2', cluster[0].keypair)
+
+    def test_create_cluster_without_keypair(self):
+        bdict = apiutils.cluster_post_data()
+        response = self.post_json('/clusters', bdict)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(202, response.status_int)
+        cluster, timeout = self.mock_cluster_create.call_args
+        # Verify keypair from ClusterTemplate is used
+        self.assertEqual('keypair1', cluster[0].keypair)
+
+    def test_create_cluster_with_multi_keypair_same_name(self):
+        bdict = apiutils.cluster_post_data()
+        self.mock_valid_os_res.side_effect = exception.Conflict('keypair2')
+        response = self.post_json('/clusters', bdict, expect_errors=True)
+        self.assertEqual('application/json', response.content_type)
+        self.assertTrue(self.mock_valid_os_res.called)
+        self.assertEqual(409, response.status_int)
+
+    def test_create_cluster_with_docker_volume_size(self):
+        bdict = apiutils.cluster_post_data()
+        bdict['docker_volume_size'] = 3
+        response = self.post_json('/clusters', bdict)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(202, response.status_int)
+        cluster, timeout = self.mock_cluster_create.call_args
+        self.assertEqual(3, cluster[0].docker_volume_size)
+
+    def test_create_cluster_with_labels(self):
+        bdict = apiutils.cluster_post_data()
+        bdict['labels'] = {'key': 'value'}
+        response = self.post_json('/clusters', bdict)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(202, response.status_int)
+        cluster, timeout = self.mock_cluster_create.call_args
+        self.assertEqual({'key': 'value'}, cluster[0].labels)
+
+    def test_create_cluster_without_docker_volume_size(self):
+        bdict = apiutils.cluster_post_data()
+        # Remove the default docker_volume_size from the cluster dict.
+        del bdict['docker_volume_size']
+        response = self.post_json('/clusters', bdict)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(202, response.status_int)
+        cluster, timeout = self.mock_cluster_create.call_args
+        # Verify docker_volume_size from ClusterTemplate is used
+        self.assertEqual(20, cluster[0].docker_volume_size)
+
+    def test_create_cluster_without_labels(self):
+        bdict = apiutils.cluster_post_data()
+        bdict.pop('labels')
+        response = self.post_json('/clusters', bdict)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(202, response.status_int)
+        cluster, timeout = self.mock_cluster_create.call_args
+        # Verify labels from ClusterTemplate is used
+        self.assertEqual({'key1': u'val1', 'key2': u'val2'}, cluster[0].labels)
+
+    def test_create_cluster_with_invalid_docker_volume_size(self):
+        invalid_values = [(-1, None), ('notanint', None),
+                          (1, 'devicemapper'), (2, 'devicemapper')]
+        for value in invalid_values:
+            bdict = apiutils.cluster_post_data(docker_volume_size=value[0],
+                                               docker_storage_driver=value[1])
+            response = self.post_json('/clusters', bdict, expect_errors=True)
+            self.assertEqual('application/json', response.content_type)
+            self.assertEqual(400, response.status_int)
+            self.assertTrue(response.json['errors'])
+
+    def test_create_cluster_with_invalid_labels(self):
+        bdict = apiutils.cluster_post_data(labels='invalid')
+        response = self.post_json('/clusters', bdict, expect_errors=True)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(400, response.status_int)
+        self.assertTrue(response.json['errors'])
+
+    def test_create_cluster_with_master_flavor_id(self):
+        bdict = apiutils.cluster_post_data()
+        bdict['master_flavor_id'] = 'm2.small'
+        response = self.post_json('/clusters', bdict)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(202, response.status_int)
+        cluster, timeout = self.mock_cluster_create.call_args
+        self.assertEqual('m2.small', cluster[0].master_flavor_id)
+
+    def test_create_cluster_without_master_flavor_id(self):
+        bdict = apiutils.cluster_post_data()
+        response = self.post_json('/clusters', bdict)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(202, response.status_int)
+        cluster, timeout = self.mock_cluster_create.call_args
+        # Verify master_flavor_id from ClusterTemplate is used
+        self.assertEqual('m1.small', cluster[0].master_flavor_id)
+
+    def test_create_cluster_with_flavor_id(self):
+        bdict = apiutils.cluster_post_data()
+        bdict['flavor_id'] = 'm2.small'
+        response = self.post_json('/clusters', bdict)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(202, response.status_int)
+        cluster, timeout = self.mock_cluster_create.call_args
+        self.assertEqual('m2.small', cluster[0].flavor_id)
+
+    def test_create_cluster_without_flavor_id(self):
+        bdict = apiutils.cluster_post_data()
+        response = self.post_json('/clusters', bdict)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(202, response.status_int)
+        cluster, timeout = self.mock_cluster_create.call_args
+        # Verify flavor_id from ClusterTemplate is used
+        self.assertEqual('m1.small', cluster[0].flavor_id)
+
 
 class TestDelete(api_base.FunctionalTest):
     def setUp(self):
