@@ -15,30 +15,38 @@
 import abc
 import mock
 from neutronclient.common import exceptions as n_exception
-from oslo_config import cfg
 import six
 
 from magnum.common import exception
-from magnum.drivers.common import template_def as cmn_tdef
+import magnum.conf
+from magnum.drivers.common import driver
+from magnum.drivers.heat import template_def as cmn_tdef
+from magnum.drivers.k8s_coreos_v1 import driver as k8s_coreos_dr
 from magnum.drivers.k8s_coreos_v1 import template_def as k8s_coreos_tdef
+from magnum.drivers.k8s_fedora_atomic_v1 import driver as k8sa_dr
 from magnum.drivers.k8s_fedora_atomic_v1 import template_def as k8sa_tdef
+from magnum.drivers.k8s_fedora_ironic_v1 import driver as k8s_i_dr
 from magnum.drivers.k8s_fedora_ironic_v1 import template_def as k8si_tdef
+from magnum.drivers.mesos_ubuntu_v1 import driver as mesos_dr
 from magnum.drivers.mesos_ubuntu_v1 import template_def as mesos_tdef
+from magnum.drivers.swarm_fedora_atomic_v1 import driver as swarm_dr
 from magnum.drivers.swarm_fedora_atomic_v1 import template_def as swarm_tdef
 from magnum.tests import base
 
 from requests import exceptions as req_exceptions
 
+CONF = magnum.conf.CONF
+
 
 class TemplateDefinitionTestCase(base.TestCase):
 
-    @mock.patch.object(cmn_tdef, 'iter_entry_points')
+    @mock.patch.object(driver, 'iter_entry_points')
     def test_load_entry_points(self, mock_iter_entry_points):
         mock_entry_point = mock.MagicMock()
         mock_entry_points = [mock_entry_point]
         mock_iter_entry_points.return_value = mock_entry_points.__iter__()
 
-        entry_points = cmn_tdef.TemplateDefinition.load_entry_points()
+        entry_points = driver.Driver.load_entry_points()
 
         for (expected_entry_point,
              (actual_entry_point, loaded_cls)) in zip(mock_entry_points,
@@ -46,77 +54,64 @@ class TemplateDefinitionTestCase(base.TestCase):
             self.assertEqual(expected_entry_point, actual_entry_point)
             expected_entry_point.load.assert_called_once_with(require=False)
 
-    def test_get_template_definitions(self):
-        defs = cmn_tdef.TemplateDefinition.get_template_definitions()
-
-        vm_atomic_k8s = defs[('vm', 'fedora-atomic', 'kubernetes')]
-        vm_coreos_k8s = defs[('vm', 'coreos', 'kubernetes')]
-
-        self.assertEqual(1, len(vm_atomic_k8s))
-        self.assertEqual(k8sa_tdef.AtomicK8sTemplateDefinition,
-                         vm_atomic_k8s['magnum_vm_atomic_k8s'])
-        self.assertEqual(1, len(vm_coreos_k8s))
-        self.assertEqual(k8s_coreos_tdef.CoreOSK8sTemplateDefinition,
-                         vm_coreos_k8s['magnum_vm_coreos_k8s'])
-
-    def test_get_vm_atomic_kubernetes_definition(self):
-        definition = cmn_tdef.TemplateDefinition.get_template_definition(
-            'vm',
-            'fedora-atomic',
-            'kubernetes')
+    @mock.patch('magnum.drivers.common.driver.Driver.get_driver')
+    def test_get_vm_atomic_kubernetes_definition(self, mock_driver):
+        mock_driver.return_value = k8sa_dr.Driver()
+        cluster_driver = driver.Driver.get_driver('vm',
+                                                  'fedora-atomic',
+                                                  'kubernetes')
+        definition = cluster_driver.get_template_definition()
 
         self.assertIsInstance(definition,
                               k8sa_tdef.AtomicK8sTemplateDefinition)
 
-    def test_get_bm_fedora_kubernetes_ironic_definition(self):
-        definition = cmn_tdef.TemplateDefinition.get_template_definition(
-            'bm',
-            'fedora',
-            'kubernetes')
+    @mock.patch('magnum.drivers.common.driver.Driver.get_driver')
+    def test_get_bm_fedora_kubernetes_ironic_definition(self, mock_driver):
+        mock_driver.return_value = k8s_i_dr.Driver()
+        cluster_driver = driver.Driver.get_driver('bm',
+                                                  'fedora',
+                                                  'kubernetes')
+        definition = cluster_driver.get_template_definition()
 
         self.assertIsInstance(definition,
                               k8si_tdef.FedoraK8sIronicTemplateDefinition)
 
-    def test_get_vm_coreos_kubernetes_definition(self):
-        definition = cmn_tdef.TemplateDefinition.get_template_definition(
-            'vm',
-            'coreos',
-            'kubernetes')
+    @mock.patch('magnum.drivers.common.driver.Driver.get_driver')
+    def test_get_vm_coreos_kubernetes_definition(self, mock_driver):
+        mock_driver.return_value = k8s_coreos_dr.Driver()
+        cluster_driver = driver.Driver.get_driver('vm', 'coreos', 'kubernetes')
+        definition = cluster_driver.get_template_definition()
 
         self.assertIsInstance(definition,
                               k8s_coreos_tdef.CoreOSK8sTemplateDefinition)
 
-    def test_get_vm_atomic_swarm_definition(self):
-        definition = cmn_tdef.TemplateDefinition.get_template_definition(
-            'vm',
-            'fedora-atomic',
-            'swarm')
+    @mock.patch('magnum.drivers.common.driver.Driver.get_driver')
+    def test_get_vm_atomic_swarm_definition(self, mock_driver):
+        mock_driver.return_value = swarm_dr.Driver()
+        cluster_driver = driver.Driver.get_driver('vm',
+                                                  'fedora-atomic',
+                                                  'swarm')
+        definition = cluster_driver.get_template_definition()
 
         self.assertIsInstance(definition,
                               swarm_tdef.AtomicSwarmTemplateDefinition)
 
-    def test_get_vm_ubuntu_mesos_definition(self):
-        definition = cmn_tdef.TemplateDefinition.get_template_definition(
-            'vm',
-            'ubuntu',
-            'mesos')
+    @mock.patch('magnum.drivers.common.driver.Driver.get_driver')
+    def test_get_vm_ubuntu_mesos_definition(self, mock_driver):
+        mock_driver.return_value = mesos_dr.Driver()
+        cluster_driver = driver.Driver.get_driver('vm',
+                                                  'ubuntu',
+                                                  'mesos')
+        definition = cluster_driver.get_template_definition()
 
         self.assertIsInstance(definition,
                               mesos_tdef.UbuntuMesosTemplateDefinition)
 
-    def test_get_definition_not_supported(self):
+    def test_get_driver_not_supported(self):
         self.assertRaises(exception.ClusterTypeNotSupported,
-                          cmn_tdef.TemplateDefinition.get_template_definition,
+                          driver.Driver.get_driver,
                           'vm', 'not_supported', 'kubernetes')
 
-    def test_get_definition_not_enabled(self):
-        cfg.CONF.set_override('enabled_definitions',
-                              ['magnum_vm_atomic_k8s'],
-                              group='cluster')
-        self.assertRaises(exception.ClusterTypeNotEnabled,
-                          cmn_tdef.TemplateDefinition.get_template_definition,
-                          'vm', 'coreos', 'kubernetes')
-
     def test_required_param_not_set(self):
         param = cmn_tdef.ParameterMapping('test', cluster_template_attr='test',
                                           required=True)
@@ -162,10 +157,7 @@ class TemplateDefinitionTestCase(base.TestCase):
         self.assertIsNone(value)
 
     def test_add_output_with_mapping_type(self):
-        definition = cmn_tdef.TemplateDefinition.get_template_definition(
-            'vm',
-            'fedora-atomic',
-            'kubernetes')
+        definition = k8sa_dr.Driver().get_template_definition()
 
         mock_args = [1, 3, 4]
         mock_kwargs = {'test': 'test'}
@@ -225,18 +217,14 @@ class BaseTemplateDefinitionTestCase(base.TestCase):
 class AtomicK8sTemplateDefinitionTestCase(BaseTemplateDefinitionTestCase):
 
     def get_definition(self):
-        return cmn_tdef.TemplateDefinition.get_template_definition(
-            'vm',
-            'fedora-atomic',
-            'kubernetes',
-        )
+        return k8sa_dr.Driver().get_template_definition()
 
     @mock.patch('magnum.common.clients.OpenStackClients')
     @mock.patch('magnum.drivers.k8s_fedora_atomic_v1.template_def'
                 '.AtomicK8sTemplateDefinition.get_discovery_url')
-    @mock.patch('magnum.drivers.common.template_def.BaseTemplateDefinition'
+    @mock.patch('magnum.drivers.heat.template_def.BaseTemplateDefinition'
                 '.get_params')
-    @mock.patch('magnum.drivers.common.template_def.TemplateDefinition'
+    @mock.patch('magnum.drivers.heat.template_def.TemplateDefinition'
                 '.get_output')
     def test_k8s_get_params(self, mock_get_output, mock_get_params,
                             mock_get_discovery_url, mock_osc_class):
@@ -260,12 +248,32 @@ class AtomicK8sTemplateDefinitionTestCase(BaseTemplateDefinitionTestCase):
 
         mock_context.auth_url = 'http://192.168.10.10:5000/v3'
         mock_context.user_name = 'fake_user'
-        mock_context.tenant = 'fake_tenant'
 
-        flannel_cidr = mock_cluster_template.labels.get('flannel_network_cidr')
-        flannel_subnet = mock_cluster_template.labels.get(
+        flannel_cidr = mock_cluster.labels.get('flannel_network_cidr')
+        flannel_subnet = mock_cluster.labels.get(
             'flannel_network_subnetlen')
-        flannel_backend = mock_cluster_template.labels.get('flannel_backend')
+        flannel_backend = mock_cluster.labels.get('flannel_backend')
+        system_pods_initial_delay = mock_cluster.labels.get(
+            'system_pods_initial_delay')
+        system_pods_timeout = mock_cluster.labels.get(
+            'system_pods_timeout')
+        admission_control_list = mock_cluster.labels.get(
+            'admission_control_list')
+        prometheus_monitoring = mock_cluster.labels.get(
+            'prometheus_monitoring')
+        grafana_admin_passwd = mock_cluster.labels.get(
+            'grafana_admin_passwd')
+        kube_dashboard_enabled = mock_cluster.labels.get(
+            'kube_dashboard_enabled')
+        docker_volume_type = mock_cluster.labels.get(
+            'docker_volume_type')
+        etcd_volume_size = mock_cluster.labels.get(
+            'etcd_volume_size')
+        kube_tag = mock_cluster.labels.get('kube_tag')
+        container_infra_prefix = mock_cluster.labels.get(
+            'container_infra_prefix')
+        availability_zone = mock_cluster.labels.get(
+            'availability_zone')
 
         k8s_def = k8sa_tdef.AtomicK8sTemplateDefinition()
 
@@ -278,21 +286,33 @@ class AtomicK8sTemplateDefinitionTestCase(BaseTemplateDefinitionTestCase):
             'flannel_network_cidr': flannel_cidr,
             'flannel_network_subnetlen': flannel_subnet,
             'flannel_backend': flannel_backend,
+            'system_pods_initial_delay': system_pods_initial_delay,
+            'system_pods_timeout': system_pods_timeout,
+            'admission_control_list': admission_control_list,
+            'prometheus_monitoring': prometheus_monitoring,
+            'grafana_admin_passwd': grafana_admin_passwd,
+            'kube_dashboard_enabled': kube_dashboard_enabled,
+            'docker_volume_type': docker_volume_type,
+            'etcd_volume_size': etcd_volume_size,
             'username': 'fake_user',
-            'tenant_name': 'fake_tenant',
             'magnum_url': mock_osc.magnum_url.return_value,
-            'region_name': mock_osc.cinder_region_name.return_value}}
+            'region_name': mock_osc.cinder_region_name.return_value,
+            'kube_tag': kube_tag,
+            'container_infra_prefix': container_infra_prefix,
+            'nodes_affinity_policy': 'soft-anti-affinity',
+            'availability_zone': availability_zone,
+        }}
         mock_get_params.assert_called_once_with(mock_context,
                                                 mock_cluster_template,
                                                 mock_cluster,
                                                 **expected_kwargs)
 
     @mock.patch('magnum.common.clients.OpenStackClients')
-    @mock.patch('magnum.drivers.common.template_def'
+    @mock.patch('magnum.drivers.heat.template_def'
                 '.BaseTemplateDefinition.get_discovery_url')
-    @mock.patch('magnum.drivers.common.template_def.BaseTemplateDefinition'
+    @mock.patch('magnum.drivers.heat.template_def.BaseTemplateDefinition'
                 '.get_params')
-    @mock.patch('magnum.drivers.common.template_def.TemplateDefinition'
+    @mock.patch('magnum.drivers.heat.template_def.TemplateDefinition'
                 '.get_output')
     def test_k8s_get_params_insecure(self, mock_get_output, mock_get_params,
                                      mock_get_discovery_url, mock_osc_class):
@@ -316,12 +336,32 @@ class AtomicK8sTemplateDefinitionTestCase(BaseTemplateDefinitionTestCase):
 
         mock_context.auth_url = 'http://192.168.10.10:5000/v3'
         mock_context.user_name = 'fake_user'
-        mock_context.tenant = 'fake_tenant'
 
-        flannel_cidr = mock_cluster_template.labels.get('flannel_network_cidr')
-        flannel_subnet = mock_cluster_template.labels.get(
+        flannel_cidr = mock_cluster.labels.get('flannel_network_cidr')
+        flannel_subnet = mock_cluster.labels.get(
             'flannel_network_subnetlen')
-        flannel_backend = mock_cluster_template.labels.get('flannel_backend')
+        flannel_backend = mock_cluster.labels.get('flannel_backend')
+        system_pods_initial_delay = mock_cluster.labels.get(
+            'system_pods_initial_delay')
+        system_pods_timeout = mock_cluster.labels.get(
+            'system_pods_timeout')
+        admission_control_list = mock_cluster.labels.get(
+            'admission_control_list')
+        prometheus_monitoring = mock_cluster.labels.get(
+            'prometheus_monitoring')
+        grafana_admin_passwd = mock_cluster.labels.get(
+            'grafana_admin_passwd')
+        kube_dashboard_enabled = mock_cluster.labels.get(
+            'kube_dashboard_enabled')
+        docker_volume_type = mock_cluster.labels.get(
+            'docker_volume_type')
+        etcd_volume_size = mock_cluster.labels.get(
+            'etcd_volume_size')
+        kube_tag = mock_cluster.labels.get('kube_tag')
+        container_infra_prefix = mock_cluster.labels.get(
+            'container_infra_prefix')
+        availability_zone = mock_cluster.labels.get(
+            'availability_zone')
 
         k8s_def = k8sa_tdef.AtomicK8sTemplateDefinition()
 
@@ -334,12 +374,24 @@ class AtomicK8sTemplateDefinitionTestCase(BaseTemplateDefinitionTestCase):
             'flannel_network_cidr': flannel_cidr,
             'flannel_network_subnetlen': flannel_subnet,
             'flannel_backend': flannel_backend,
+            'system_pods_initial_delay': system_pods_initial_delay,
+            'system_pods_timeout': system_pods_timeout,
+            'admission_control_list': admission_control_list,
+            'prometheus_monitoring': prometheus_monitoring,
+            'grafana_admin_passwd': grafana_admin_passwd,
+            'kube_dashboard_enabled': kube_dashboard_enabled,
+            'docker_volume_type': docker_volume_type,
+            'etcd_volume_size': etcd_volume_size,
             'username': 'fake_user',
-            'tenant_name': 'fake_tenant',
             'magnum_url': mock_osc.magnum_url.return_value,
             'region_name': mock_osc.cinder_region_name.return_value,
             'loadbalancing_protocol': 'HTTP',
-            'kubernetes_port': 8080}}
+            'kubernetes_port': 8080,
+            'kube_tag': kube_tag,
+            'container_infra_prefix': container_infra_prefix,
+            'nodes_affinity_policy': 'soft-anti-affinity',
+            'availability_zone': availability_zone,
+        }}
         mock_get_params.assert_called_once_with(mock_context,
                                                 mock_cluster_template,
                                                 mock_cluster,
@@ -391,9 +443,9 @@ class AtomicK8sTemplateDefinitionTestCase(BaseTemplateDefinitionTestCase):
 
     @mock.patch('requests.get')
     def test_k8s_get_discovery_url(self, mock_get):
-        cfg.CONF.set_override('etcd_discovery_service_endpoint_format',
-                              'http://etcd/test?size=%(size)d',
-                              group='cluster')
+        CONF.set_override('etcd_discovery_service_endpoint_format',
+                          'http://etcd/test?size=%(size)d',
+                          group='cluster')
         expected_discovery_url = 'http://etcd/token'
         mock_resp = mock.MagicMock()
         mock_resp.text = expected_discovery_url
@@ -411,9 +463,9 @@ class AtomicK8sTemplateDefinitionTestCase(BaseTemplateDefinitionTestCase):
 
     @mock.patch('requests.get')
     def test_k8s_get_discovery_url_fail(self, mock_get):
-        cfg.CONF.set_override('etcd_discovery_service_endpoint_format',
-                              'http://etcd/test?size=%(size)d',
-                              group='cluster')
+        CONF.set_override('etcd_discovery_service_endpoint_format',
+                          'http://etcd/test?size=%(size)d',
+                          group='cluster')
         mock_get.side_effect = req_exceptions.RequestException()
         mock_cluster = mock.MagicMock()
         mock_cluster.master_count = 10
@@ -444,12 +496,9 @@ class AtomicK8sTemplateDefinitionTestCase(BaseTemplateDefinitionTestCase):
             k8sa_tdef.AtomicK8sTemplateDefinition().get_discovery_url,
             fake_cluster)
 
-    def _test_update_outputs_api_address(self, coe, params, tls=True):
+    def _test_update_outputs_api_address(self, template_definition,
+                                         params, tls=True):
 
-        definition = cmn_tdef.TemplateDefinition.get_template_definition(
-            'vm',
-            'fedora-atomic',
-            coe)
         expected_api_address = '%(protocol)s://%(address)s:%(port)s' % params
 
         outputs = [
@@ -463,8 +512,8 @@ class AtomicK8sTemplateDefinitionTestCase(BaseTemplateDefinitionTestCase):
         mock_cluster_template = mock.MagicMock()
         mock_cluster_template.tls_disabled = tls
 
-        definition.update_outputs(mock_stack, mock_cluster_template,
-                                  mock_cluster)
+        template_definition.update_outputs(mock_stack, mock_cluster_template,
+                                           mock_cluster)
 
         self.assertEqual(expected_api_address, mock_cluster.api_address)
 
@@ -477,7 +526,9 @@ class AtomicK8sTemplateDefinitionTestCase(BaseTemplateDefinitionTestCase):
             'address': address,
             'port': port,
         }
-        self._test_update_outputs_api_address('kubernetes', params)
+
+        template_definition = k8sa_tdef.AtomicK8sTemplateDefinition()
+        self._test_update_outputs_api_address(template_definition, params)
 
     def test_update_swarm_outputs_api_address(self):
         address = 'updated_address'
@@ -488,7 +539,9 @@ class AtomicK8sTemplateDefinitionTestCase(BaseTemplateDefinitionTestCase):
             'address': address,
             'port': port,
         }
-        self._test_update_outputs_api_address('swarm', params)
+
+        template_definition = swarm_tdef.AtomicSwarmTemplateDefinition()
+        self._test_update_outputs_api_address(template_definition, params)
 
     def test_update_k8s_outputs_if_cluster_template_is_secure(self):
         address = 'updated_address'
@@ -499,7 +552,9 @@ class AtomicK8sTemplateDefinitionTestCase(BaseTemplateDefinitionTestCase):
             'address': address,
             'port': port,
         }
-        self._test_update_outputs_api_address('kubernetes', params, tls=False)
+        template_definition = k8sa_tdef.AtomicK8sTemplateDefinition()
+        self._test_update_outputs_api_address(template_definition, params,
+                                              tls=False)
 
     def test_update_swarm_outputs_if_cluster_template_is_secure(self):
         address = 'updated_address'
@@ -510,14 +565,13 @@ class AtomicK8sTemplateDefinitionTestCase(BaseTemplateDefinitionTestCase):
             'address': address,
             'port': port,
         }
-        self._test_update_outputs_api_address('swarm', params, tls=False)
 
-    def _test_update_outputs_none_api_address(self, coe, params, tls=True):
+        template_definition = swarm_tdef.AtomicSwarmTemplateDefinition()
+        self._test_update_outputs_api_address(template_definition, params,
+                                              tls=False)
 
-        definition = cmn_tdef.TemplateDefinition.get_template_definition(
-            'vm',
-            'fedora-atomic',
-            coe)
+    def _test_update_outputs_none_api_address(self, template_definition,
+                                              params, tls=True):
 
         outputs = [
             {"output_value": params['address'],
@@ -531,8 +585,8 @@ class AtomicK8sTemplateDefinitionTestCase(BaseTemplateDefinitionTestCase):
         mock_cluster_template = mock.MagicMock()
         mock_cluster_template.tls_disabled = tls
 
-        definition.update_outputs(mock_stack, mock_cluster_template,
-                                  mock_cluster)
+        template_definition.update_outputs(mock_stack, mock_cluster_template,
+                                           mock_cluster)
 
         self.assertEqual('none_api_address', mock_cluster.api_address)
 
@@ -544,7 +598,9 @@ class AtomicK8sTemplateDefinitionTestCase(BaseTemplateDefinitionTestCase):
             'address': None,
             'port': port,
         }
-        self._test_update_outputs_none_api_address('kubernetes', params)
+
+        template_definition = k8sa_tdef.AtomicK8sTemplateDefinition()
+        self._test_update_outputs_none_api_address(template_definition, params)
 
     def test_update_swarm_outputs_none_api_address(self):
         protocol = 'tcp'
@@ -554,7 +610,8 @@ class AtomicK8sTemplateDefinitionTestCase(BaseTemplateDefinitionTestCase):
             'address': None,
             'port': port,
         }
-        self._test_update_outputs_none_api_address('swarm', params)
+        template_definition = swarm_tdef.AtomicSwarmTemplateDefinition()
+        self._test_update_outputs_none_api_address(template_definition, params)
 
     def test_update_outputs_master_address(self):
         self._test_update_outputs_server_addrtess(
@@ -590,11 +647,7 @@ class AtomicK8sTemplateDefinitionTestCase(BaseTemplateDefinitionTestCase):
 class FedoraK8sIronicTemplateDefinitionTestCase(base.TestCase):
 
     def get_definition(self):
-        return cmn_tdef.TemplateDefinition.get_template_definition(
-            'bm',
-            'fedora',
-            'kubernetes'
-        )
+        return k8s_i_dr.Driver().get_template_definition()
 
     def assert_neutron_find(self, mock_neutron_v20_find,
                             osc, cluster_template):
@@ -700,9 +753,9 @@ class AtomicSwarmTemplateDefinitionTestCase(base.TestCase):
     @mock.patch('magnum.common.clients.OpenStackClients')
     @mock.patch('magnum.drivers.swarm_fedora_atomic_v1.template_def'
                 '.AtomicSwarmTemplateDefinition.get_discovery_url')
-    @mock.patch('magnum.drivers.common.template_def.BaseTemplateDefinition'
+    @mock.patch('magnum.drivers.heat.template_def.BaseTemplateDefinition'
                 '.get_params')
-    @mock.patch('magnum.drivers.common.template_def.TemplateDefinition'
+    @mock.patch('magnum.drivers.heat.template_def.TemplateDefinition'
                 '.get_output')
     def test_swarm_get_params(self, mock_get_output, mock_get_params,
                               mock_get_discovery_url, mock_osc_class):
@@ -724,11 +777,14 @@ class AtomicSwarmTemplateDefinitionTestCase(base.TestCase):
         mock_context.user_name = 'fake_user'
         mock_context.tenant = 'fake_tenant'
 
-        flannel_cidr = mock_cluster_template.labels.get('flannel_network_cidr')
-        flannel_subnet = mock_cluster_template.labels.get(
+        docker_volume_type = mock_cluster.labels.get(
+            'docker_volume_type')
+        flannel_cidr = mock_cluster.labels.get('flannel_network_cidr')
+        flannel_subnet = mock_cluster.labels.get(
             'flannel_network_subnetlen')
-        flannel_backend = mock_cluster_template.labels.get('flannel_backend')
-        rexray_preempt = mock_cluster_template.labels.get('rexray_preempt')
+        flannel_backend = mock_cluster.labels.get('flannel_backend')
+        rexray_preempt = mock_cluster.labels.get('rexray_preempt')
+        swarm_strategy = mock_cluster.labels.get('swarm_strategy')
 
         swarm_def = swarm_tdef.AtomicSwarmTemplateDefinition()
 
@@ -741,7 +797,10 @@ class AtomicSwarmTemplateDefinitionTestCase(base.TestCase):
             'flannel_backend': flannel_backend,
             'flannel_network_subnetlen': flannel_subnet,
             'auth_url': 'http://192.168.10.10:5000/v3',
-            'rexray_preempt': rexray_preempt}}
+            'rexray_preempt': rexray_preempt,
+            'swarm_strategy': swarm_strategy,
+            'docker_volume_type': docker_volume_type,
+            'nodes_affinity_policy': 'soft-anti-affinity'}}
         mock_get_params.assert_called_once_with(mock_context,
                                                 mock_cluster_template,
                                                 mock_cluster,
@@ -793,9 +852,9 @@ class AtomicSwarmTemplateDefinitionTestCase(base.TestCase):
 
     @mock.patch('requests.get')
     def test_swarm_get_discovery_url(self, mock_get):
-        cfg.CONF.set_override('etcd_discovery_service_endpoint_format',
-                              'http://etcd/test?size=%(size)d',
-                              group='cluster')
+        CONF.set_override('etcd_discovery_service_endpoint_format',
+                          'http://etcd/test?size=%(size)d',
+                          group='cluster')
         expected_discovery_url = 'http://etcd/token'
         mock_resp = mock.MagicMock()
         mock_resp.text = expected_discovery_url
@@ -868,31 +927,31 @@ class AtomicSwarmTemplateDefinitionTestCase(base.TestCase):
 class UbuntuMesosTemplateDefinitionTestCase(base.TestCase):
 
     @mock.patch('magnum.common.clients.OpenStackClients')
-    @mock.patch('magnum.drivers.common.template_def.BaseTemplateDefinition'
+    @mock.patch('magnum.drivers.heat.template_def.BaseTemplateDefinition'
                 '.get_params')
-    @mock.patch('magnum.drivers.common.template_def.TemplateDefinition'
+    @mock.patch('magnum.drivers.heat.template_def.TemplateDefinition'
                 '.get_output')
     def test_mesos_get_params(self, mock_get_output, mock_get_params,
                               mock_osc_class):
         mock_context = mock.MagicMock()
         mock_context.auth_url = 'http://192.168.10.10:5000/v3'
         mock_context.user_name = 'mesos_user'
-        mock_context.tenant = 'admin'
+        mock_context.project_id = 'admin'
         mock_context.domain_name = 'domainname'
         mock_cluster_template = mock.MagicMock()
         mock_cluster_template.tls_disabled = False
-        rexray_preempt = mock_cluster_template.labels.get('rexray_preempt')
-        mesos_slave_isolation = mock_cluster_template.labels.get(
-            'mesos_slave_isolation')
-        mesos_slave_work_dir = mock_cluster_template.labels.get(
-            'mesos_slave_work_dir')
-        mesos_slave_image_providers = mock_cluster_template.labels.get(
-            'image_providers')
-        mesos_slave_executor_env_variables = mock_cluster_template.labels.get(
-            'mesos_slave_executor_env_variables')
         mock_cluster = mock.MagicMock()
         mock_cluster.uuid = '5d12f6fd-a196-4bf0-ae4c-1f639a523a52'
         del mock_cluster.stack_id
+        rexray_preempt = mock_cluster.labels.get('rexray_preempt')
+        mesos_slave_isolation = mock_cluster.labels.get(
+            'mesos_slave_isolation')
+        mesos_slave_work_dir = mock_cluster.labels.get(
+            'mesos_slave_work_dir')
+        mesos_slave_image_providers = mock_cluster.labels.get(
+            'image_providers')
+        mesos_slave_executor_env_variables = mock_cluster.labels.get(
+            'mesos_slave_executor_env_variables')
         mock_osc = mock.MagicMock()
         mock_osc.cinder_region_name.return_value = 'RegionOne'
         mock_osc_class.return_value = mock_osc
